var EventEmitter = require('event-emitter')
var async = require('async')

var Loader = function(opt) {
	this.emitter = new EventEmitter()
	// default
	this.size = opt.size || 1024
}
Loader.prototype.load = function(file) {
	var that = this
	that.offset = 0
	that.index = 0
	that.file = file
	async.waterfall([
		function(callback) {
			that.emitter.emit('start',callback)
		},
		// upload parallel
		function(callback) {
			var chunkDone = 0,chunkLength = 0
			that._read(
				function(err,chunk,index,attr) {
					if(err) {
						callback(err)
						return
					}
					async.waterfall([
						function(callback) {
							that.emitter.emit(
								'chunk',
								{
									index:index,
									data:chunk,
									length:attr.length,
									offset:attr.offset
								},
								callback
							)
						}
					],function(err) {
						if(err) {
							callback(err)
							return
						}
						chunkDone++
						if(chunkDone === chunkLength) {
							callback()
						}
					})
				},
				function(err,length) {
					chunkLength = length
					if(chunkDone === chunkLength) {
						callback()
					}
				}
			)
		}
	],function(err) {
		if(err) {
			that.emitter.emit('error',err)
			return
		}
		that.emitter.emit('complete',that)
	})
}
Loader.prototype.on = function(type,listener) {
	this.emitter.on(type,listener)
	return this
}
Loader.prototype._arrayBufferToBase64 = function(buffer) {
	var binary = ''
	var bytes = new Uint8Array( buffer )
	var len = bytes.byteLength
	for (var i = 0; i < len; i++) {
		binary += String.fromCharCode( bytes[ i ] )
	}
	return window.btoa( binary )
}
Loader.prototype._read = function(chunkCallback,callback) {
	var that = this
	var r = new FileReader()
	var blob = that.file.slice(that.offset, that.size*1024 + that.offset)
	r.onload = function(evt) {
		var offset = that.offset
		if (evt.target.error === null) {
			that.offset += blob.size
			var data = evt.target.result.substr('data:;base64,'.length)
			chunkCallback(null,data,that.index,{
				offset:offset,
				length:blob.size
			})
		} else {
			chunkCallback(evt.target.error)
			return
		}
		that.index++
		if(that.offset >= that.file.size) {
			if(callback) callback(null,that.index)
			return
		}
		that._read(chunkCallback,callback)
	}
	r.readAsDataURL(blob)
}

module.exports = Loader
